#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Calculate The price at which the current bar must close in order for a moving 
	/// average cross to occur.
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculate The price at which the current bar must close in order for a moving average cross to occur.")]
    public class SMACrossPredictor : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int fastLength = 10; // Default setting for FastLength
        private int slowLength = 25; // Default setting for SlowLength
        // User defined variables (add any user defined variables below)
		private SMA fast, slow;
		private DataSeries cross = null;
        #endregion

        protected override void Initialize()
        {            
			Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "Fast SMA"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Slow SMA"));
			
            Overlay				= true;
			
			cross = new DataSeries(this);
        }
		
		protected override void OnStartUp()
		{
			fast=SMA(Input, fastLength);
			slow=SMA(Input, slowLength);
		}

        protected override void OnBarUpdate()
        {            						
			double lastFast = fast[0] * Math.Min(CurrentBar, fastLength);
			double lastSlow = slow[0] * Math.Min(CurrentBar, slowLength);
		
			if (CurrentBar<Math.Max(fastLength,slowLength))
				return;
			
			double f = fast[0];
			double s = slow[0];
			double price = Input[0];
			
			if (fast[0]<slow[0])
			{		
				price-=TickSize;
				while (f<s)
				{										
					f=(lastFast + price - Input[fastLength]) / Math.Min(CurrentBar, fastLength);
					s=(lastSlow + price - Input[slowLength]) / Math.Min(CurrentBar, slowLength);					
					price+=TickSize;					
				}				
			}
			else
			{
				price+=TickSize;
				while (f>s)
				{
					f=(lastFast + price - Input[fastLength]) / Math.Min(CurrentBar, fastLength);
					s=(lastSlow + price - Input[slowLength]) / Math.Min(CurrentBar, slowLength);					
					price-=TickSize;
				}				
			}	
			
			Fast.Set(fast[0]);
			Slow.Set(slow[0]);
			Cross.Set(price);			
        }
		
		public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
        {
            if (Bars == null || ChartControl == null)
                return;

            for (int idx = FirstBarIndexPainted; idx <= LastBarIndexPainted; idx++)
            {
                double tmpHigh = Bars.GetHigh(idx);
                double tmpLow = Bars.GetLow(idx);

                if (tmpHigh != 0 && tmpHigh > max)
                    max = tmpHigh;
                if (tmpLow != 0 && tmpLow < min)
                    min = tmpLow;
            }
			max=Math.Max(max,Cross.Get(LastBarIndexPainted));
			min=Math.Min(min,Cross.Get(LastBarIndexPainted));
        }
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			base.Plot(graphics,bounds,min,max);
			
			int lastBar = LastBarIndexPainted;
			double f = Fast.Get(lastBar);
			double s = Slow.Get(lastBar);	
			double cross = Cross.Get(lastBar);
			
			Color clr = f>s ? Color.Green : Color.Red;
			
			Pen pen = new Pen(clr);
			SolidBrush brush = new SolidBrush(clr);
			
			int x2 = bounds.Right;
			int x1 = (int)Math.Round((decimal)x2*2/3);
			int y = ChartControl.GetYByValue(this,cross);
			
			StringFormat sf = new StringFormat();
			sf.Alignment=StringAlignment.Near;
			sf.LineAlignment=f>s ? StringAlignment.Near : StringAlignment.Far;
			
			Font font = ChartControl!=null ? ChartControl.Font : new Font("Arial",12,FontStyle.Regular);
						
			graphics.DrawLine(pen,x1,y,x2,y);						
			
			string str = "Moving Averages will cross if current bar closes at " + cross.ToString();
			graphics.DrawString(str,font,brush,x1,y,sf);
		}
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Fast
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Slow
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Cross
        {
            get { return cross; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int FastLength
        {
            get { return fastLength; }
            set { fastLength = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int SlowLength
        {
            get { return slowLength; }
            set { slowLength = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SMACrossPredictor[] cacheSMACrossPredictor = null;

        private static SMACrossPredictor checkSMACrossPredictor = new SMACrossPredictor();

        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        public SMACrossPredictor SMACrossPredictor(int fastLength, int slowLength)
        {
            return SMACrossPredictor(Input, fastLength, slowLength);
        }

        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        public SMACrossPredictor SMACrossPredictor(Data.IDataSeries input, int fastLength, int slowLength)
        {
            if (cacheSMACrossPredictor != null)
                for (int idx = 0; idx < cacheSMACrossPredictor.Length; idx++)
                    if (cacheSMACrossPredictor[idx].FastLength == fastLength && cacheSMACrossPredictor[idx].SlowLength == slowLength && cacheSMACrossPredictor[idx].EqualsInput(input))
                        return cacheSMACrossPredictor[idx];

            lock (checkSMACrossPredictor)
            {
                checkSMACrossPredictor.FastLength = fastLength;
                fastLength = checkSMACrossPredictor.FastLength;
                checkSMACrossPredictor.SlowLength = slowLength;
                slowLength = checkSMACrossPredictor.SlowLength;

                if (cacheSMACrossPredictor != null)
                    for (int idx = 0; idx < cacheSMACrossPredictor.Length; idx++)
                        if (cacheSMACrossPredictor[idx].FastLength == fastLength && cacheSMACrossPredictor[idx].SlowLength == slowLength && cacheSMACrossPredictor[idx].EqualsInput(input))
                            return cacheSMACrossPredictor[idx];

                SMACrossPredictor indicator = new SMACrossPredictor();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.FastLength = fastLength;
                indicator.SlowLength = slowLength;
                Indicators.Add(indicator);
                indicator.SetUp();

                SMACrossPredictor[] tmp = new SMACrossPredictor[cacheSMACrossPredictor == null ? 1 : cacheSMACrossPredictor.Length + 1];
                if (cacheSMACrossPredictor != null)
                    cacheSMACrossPredictor.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSMACrossPredictor = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SMACrossPredictor SMACrossPredictor(int fastLength, int slowLength)
        {
            return _indicator.SMACrossPredictor(Input, fastLength, slowLength);
        }

        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        public Indicator.SMACrossPredictor SMACrossPredictor(Data.IDataSeries input, int fastLength, int slowLength)
        {
            return _indicator.SMACrossPredictor(input, fastLength, slowLength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SMACrossPredictor SMACrossPredictor(int fastLength, int slowLength)
        {
            return _indicator.SMACrossPredictor(Input, fastLength, slowLength);
        }

        /// <summary>
        /// Calculate The price at which the current bar must close in order for a moving average cross to occur.
        /// </summary>
        /// <returns></returns>
        public Indicator.SMACrossPredictor SMACrossPredictor(Data.IDataSeries input, int fastLength, int slowLength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SMACrossPredictor(input, fastLength, slowLength);
        }
    }
}
#endregion
